from typing import Literal
def b_get_norm(
    lst: list,
    mode: Literal['min-max'] = 'min-max',
    ndigits: int | None = None
) -> tuple[list, float, float]:

    if mode =='min-max':
        minimum = min(lst)
        maximum = max(lst)
        if ndigits is None:
            result = [(x-minimum)/(maximum-minimum) for x in lst]
        else:
            result = [round((x-minimum)/(maximum-minimum), ndigits) for x in lst]
        return (result, minimum, maximum)


if __name__ == '__main__':
    lst = [1, 2, 3, 4, 5]
    result = b_get_norm(lst)
    print(result)


from .grid_train import grid_trains_1d, grid_trains_2d
from .get_device import get_device
from .get_params import get_params
from .get_flops import get_flops

all = [
    'grid_trains_1d', 'grid_trains_2d',
    'get_device', 'get_params', 'get_flops',
]



import numpy as np
from torch.utils.data import Dataset, DataLoader
import torch

class MyDataset(Dataset):
    def __init__(self, data, label):
        self.data = data
        self.label = label
    def __len__(self):
        return len(self.data)
    def __getitem__(self, index):
        return self.data[index], self.label[index]


def merge_dataset(*args: Dataset):
    datas_labels = []

    for dataset in args:
        if not isinstance(dataset, Dataset):
            raise TypeError("dataset must be an instance of Dataset")
        length = len(dataset)
        for i in range(length):
            data, label = dataset.__getitem__(i)
            datas_labels.append([data, label])

    # 根据label对datas_labels排序
    datas_labels = sorted(datas_labels, key=lambda x: x[1]) # 从小到大
    # 拆分开
    datas, labels = zip(*datas_labels)

    datas = torch.stack(datas, dim=0)
    labels = torch.tensor(labels)

    my_dataset = MyDataset(datas, labels)
    return my_dataset







import numpy as np
import seaborn as sns
import matplotlib.pyplot as plt
from sklearn.metrics import confusion_matrix


def draw_confusion_matrix1(y_true, y_pred):
    # 计算混淆矩阵
    labels = sorted(set(y_true))  # 确保标签按顺序
    cm = confusion_matrix(y_true, y_pred, labels=labels)
    labels = [i for i in range(len(cm))]

    # 绘制热力图
    plt.figure(figsize=(7, 6))
    ax = sns.heatmap(cm, annot=True, fmt="d", cmap="Reds", xticklabels=labels, yticklabels=labels)
    ax.xaxis.set_ticks_position("top")  # 将 X 轴的刻度移动到顶部
    ax.xaxis.set_label_position("top")  # 将 X 轴的标签移动到顶部
    plt.xlabel("Predicted Label", fontsize=14)
    plt.ylabel("True Label", fontsize=14)

    return plt

def draw_confusion_matrix2(cm):
    cm = np.array(cm)
    labels = [i for i in range(len(cm))]

    # 绘制热力图
    plt.figure(figsize=(7, 6))
    ax = sns.heatmap(cm, annot=True, fmt="d", cmap="Reds", xticklabels=labels, yticklabels=labels)
    ax.xaxis.set_ticks_position("top")  # 将 X 轴的刻度移动到顶部
    ax.xaxis.set_label_position("top")  # 将 X 轴的标签移动到顶部
    plt.xlabel("Predicted Label", fontsize=14)
    plt.ylabel("True Label", fontsize=14)

    return plt

if __name__ == '__main__':
    cm = np.array(
        [[140, 0, 5, 0, 0, 0, 2, 0, 1, 1, 0, 0],
         [1, 155, 3, 0, 0, 0, 1, 3, 0, 0, 1, 0],
         [2, 0, 129, 0, 0, 0, 2, 0, 1, 0, 0, 1],
         [0, 1, 2, 110, 1, 1, 4, 3, 0, 1, 0, 1],
         [0, 0, 3, 2, 102, 2, 0, 1, 0, 0, 0, 1],
         [1, 1, 7, 1, 2, 98, 2, 6, 1, 0, 0, 0],
         [1, 2, 9, 0, 1, 3, 124, 1, 1, 0, 0, 3],
         [3, 0, 4, 0, 4, 2, 4, 127, 1, 0, 0, 2],
         [1, 1, 2, 1, 1, 0, 2, 6, 97, 2, 13, 2],
         [0, 2, 3, 0, 1, 0, 0, 1, 5, 100, 7, 5],
         [0, 5, 0, 1, 1, 1, 0, 1, 9, 5, 97, 1],
         [1, 4, 4, 0, 1, 2, 1, 1, 3, 3, 2, 116]]
    )
    draw_confusion_matrix2(cm)


class BStopByByzh:
    def __init__(self, loss_rounds=20, loss_target=0.001, acc_rounds=50, acc_target=1):
        self.loss_rounds = loss_rounds
        self.loss_target = loss_target
        self.acc_rounds = acc_rounds
        self.acc_target = acc_target

        self.loss_list = [114514 for _ in range(loss_rounds)]
        self.acc_list = [0 for _ in range(acc_rounds)]

        self.loss_average_list = [114514 for _ in range(loss_rounds)]
        self.acc_average_list = [0 for _ in range(acc_rounds)]

        self.loss_avg_trend = None
        self.acc_avg_trend = None

        self.output = ""
    def __call__(self, train_loss, val_acc):
        if train_loss <= self.loss_target:
            self.output = f"train_loss小于{self.loss_target}"
            return True
        if val_acc >= self.acc_target:
            self.output = f"val_acc大于等于{self.acc_target}"
            return True
        self.update(self.loss_list, train_loss)
        self.update(self.acc_list, val_acc)

        self.update(self.loss_average_list, sum(self.loss_list) / self.loss_rounds)
        self.update(self.acc_average_list, sum(self.acc_list) / self.acc_rounds)

        self.loss_avg_trend = self.get_trend(self.loss_average_list)
        self.acc_avg_trend = self.get_trend(self.acc_average_list)
        if sum(self.loss_avg_trend) > 0:
            self.output = f"loss_avg_trend:{self.loss_avg_trend}"
            return True
        if sum(self.acc_avg_trend) < 0:
            self.output = f"acc_avg_trend:{self.acc_avg_trend}"
            return True
        return False

    def update(self, lst, train_loss):
        lst.append(train_loss)
        lst.pop(0)

    def get_trend(self, lst):
        temp = lst.copy()
        x = temp[0]
        for i in range(len(temp)):
            if temp[i] < x:
                temp[i] = -1
            elif temp[i] > x:
                temp[i] = 1
            else:
                temp[i] = 0
        return temp


import numpy as np
import torch
def data_standard1d(source_data, target_datas: list):

    if type(source_data) == torch.Tensor:
        mean = torch.mean(source_data, dim=(0, 2), keepdim=True)
        std = torch.std(source_data, dim=(0, 2), keepdim=True) + 1e-8
    # elif type(source_data) == np.ndarray:
    else:
        mean = np.mean(source_data, axis=(0, 2), keepdims=True)
        std = np.std(source_data, axis=(0, 2), keepdims=True) + 1e-8

    results = []
    for target_data in target_datas:
        target_data = (target_data - mean) / std
        results.append(target_data)

    return results


def get_params(model):
    '''
    获取模型参数数量params
    :param model:
    :return:
    '''
    params = sum(p.numel() for p in model.parameters() if p.requires_grad)
    return params

if __name__ == '__main__':

    import torchvision.models as models
    # 实例化模型
    model = models.resnet18()
    # 打印参数数量
    print(get_params(model))
